﻿using System;
using System.Collections.Generic;

namespace HeuristicLab.Problems.ProgramSynthesis {
  using HeuristicLab.Core;

  public class EnabledExpressionsChangedEventArgs : EventArgs {
    public IEnumerable<string> AddedExpressions { get; private set; }
    public IEnumerable<string> RemovedExpressions { get; private set; }

    public EnabledExpressionsChangedEventArgs(IEnumerable<string> addedExpressions, IEnumerable<string> removedExpressions) {
      AddedExpressions = addedExpressions;
      RemovedExpressions = removedExpressions;
    }
  }

  public interface IReadOnlyExpressionsConfiguration : INamedItem {
    int InExpressionCount { get; }


    IReadOnlyList<string> EnabledExpressions { get; }
    IReadOnlyDictionary<StackTypes, int> ExpressionsPerStackCount { get; }
  }

  public interface IExpressionsConfiguration : IReadOnlyExpressionsConfiguration {
    event EventHandler<EnabledExpressionsChangedEventArgs> EnabledExpressionsChanged;

    void EnableStack(StackTypes types, bool enableDependencies = false);
    void DisableStack(StackTypes type, bool enableDependencies = false);
    void SetStack(StackTypes type, bool state, bool setDependencies = false);
    void EnableExpressionOfStack(StackTypes types);
    void DisableExpressionsOfStack(StackTypes types);
    void EnableExpression(string name);
    void DisableExpression(string name);
    void SetExpression(string name, bool state);
    void SetExpression<T>(bool state) where T : Expression;
    void EnableExpression<T>() where T : Expression;
    void DisableExpression<T>() where T : Expression;
  }
}
